/*==============================================================================

       misc_dlg.cpp - Miscellaneous Dialogs

       Copyright  2001 Pankhurst Algorithmics - All Rights Reserved
       visit www.ebmDevMag.com for permissions/restrictions on use

==============================================================================*/
#include <stdlib.h>
#include <string.h>
extern "C"
{
#include <l4/kernel.h>
#include <l4/types.h>
#include <lcd.h>
}
#include "gui.h"
#include "gui_pkg.h"

#include "misc_dlg.h"
/*==============================================================================

       CMsgBox - class to handle text dialog

==============================================================================*/
class CMsgBox : public CWindow
{

    private:

      enum
      {
        ID_WINDOW=100,
        ID_BUTTON_A,
        ID_BUTTON_B,
        ID_BUTTON_C,
        ID_MSG,
      };

      S32 MsgHandler(MSG_TYPE type, CViewable *from, S32 data);

    public:

      static int Choice;

      CMsgBox(const char *title,
              const char *caption,
              const char *buttonAText,
              const char *buttonBText,
              const char *buttonCText);

};
//------------------------------------------------------------------------------
/*static*/ int CMsgBox::Choice;
//------------------------------------------------------------------------------
CMsgBox::CMsgBox( const char *title,
                  const char *text_message,
                  const char *buttonAText,
                  const char *buttonBText,
                  const char *buttonCText )
 : CWindow(ID_WINDOW,5,5,LCD_QueryWidth()-10,LCD_QueryHeight()-10,title,TRUE,FALSE,FALSE)
{
  RECT rect;
  GetUsableRect(&rect); // get interior dimensions to calculate spacing
  //
  int gap=5; // nice visual gap between items, edge
  int buttonHeight=20; // button sizes - needed for estimating text height
  // set up text display - turn off editing so we get formatted text/# lines
  // we start with a full window box, and then shrink it to fit the text lines
	CTextEdit *textBox=new CTextEdit(ID_MSG,
                                   rect.width-gap-gap,
                                   rect.height-gap-gap,
                                   TEXTOPTION_NOEDIT|TEXTOPTION_NOUNDERLINE );
	textBox->SetFont( GUI_GetSystemFont() ); // system font preferable for this dialog
	textBox->SetText(text_message); // display caption
	int totalTextSize=textBox->GetMaxHeight(); // total message height in pixels
  // get max possible height - text, buttons, and gaps between them
  int maxTextSize=rect.height-buttonHeight-3*gap;
  maxTextSize-=maxTextSize%GUI_FontHeight(GUI_GetSystemFont()); // make it even # of lines high
  if (maxTextSize<totalTextSize) // can we fit it all onscreen, including buttons?
    totalTextSize=maxTextSize; // no, so limit to max viewable
  // now we resize for the perfect text box
	textBox->SetHeight(totalTextSize);
  // we have the text box sized to fit the text, so we place buttons just below
  // arrange to center whatever buttons we are presenting
  int i=3; // figure out how many buttons to display
  if ( NULL==buttonCText )
    --i;
  if ( NULL==buttonBText )
    --i;
  int buttonWidth=rect.width/i-gap; // get reasonable size for buttons
  if (buttonWidth>55) // ...but not too large
    buttonWidth=55;
  int x=(rect.width-buttonWidth*i-gap*(i-1))/2, y=totalTextSize+2*gap, dx=buttonWidth+gap;
  // draw 1-3 buttons as needed
  AddChild( new CButton(ID_BUTTON_A,buttonWidth,buttonHeight,buttonAText), x,y );
  if ( i>1 )
  {
    x+=dx;
    AddChild( new CButton(ID_BUTTON_B,buttonWidth,buttonHeight,buttonBText), x,y );
  }
  if ( i>2 )
  {
    x+=dx;
    AddChild( new CButton(ID_BUTTON_C,buttonWidth,buttonHeight,buttonCText), x,y );
  }
  // now we resize and reposition the window to appear centered - we need to compare
  // the usable space to outside window dimensions so we take into account frame
  // sizes (which we cannot assume will remain the same size always)
  int frameVerticalSize=GetHeight()-rect.height;
  int newUsableHeight=y+buttonHeight+gap; // button position plus height and gap
  int newFullHeight=newUsableHeight+frameVerticalSize;
  SetHeight(newFullHeight);
  // now we center vertically
  SetY((LCD_QueryHeight()-newFullHeight)/2);
  // add text box, and we are ready to go
	AddChild(textBox,gap,gap);
}
//------------------------------------------------------------------------------
S32 CMsgBox::MsgHandler(MSG_TYPE type, CViewable *from, S32 data)
{
	if (type == MSG_BUTTON_SELECT)
  {
    switch (from->GetID())
    {
      case ID_BUTTON_A:
        Choice=0;
        break;
      case ID_BUTTON_B:
        Choice=1;
        break;
      case ID_BUTTON_C:
        Choice=2;
        break;
    }
  	Close(); // all buttons close the dialog
	  return 1;
  }
  return CWindow::MsgHandler(type, from, data);
}
/*==============================================================================

       wrapper function for message dialog

==============================================================================*/
int MessageBox(const char *caption,
               const char *buttonAText,
               const char *buttonBText,
               const char *buttonCText)
{
  // calls dialog to display text and up to three buttons - returns index of
  // choice: 0=buttonA, 1=buttonB, 2=buttonC
	GUI_EventLoop( new CMsgBox(0,caption,buttonAText,buttonBText,buttonCText) );
  return CMsgBox::Choice;
}
/*==============================================================================


       CInputBox - class to handle text entry

==============================================================================*/
class CInputBox : public CWindow
{

  private:

    enum
    {
      ID_WINDOW=100,
      ID_TEXTEDIT,
      ID_BUTTON_OK,
      ID_BUTTON_CANCEL,
      ID_BUTTON_KEYBOARD,
    };

    CTextEdit *m_textEdit;
    char*m_text;
    int  m_maxLen;

	  S32 MsgHandler(MSG_TYPE type, CViewable *from, S32 data);
	  void Draw(void);

  public:

    static bool Valid;

    CInputBox(const char *caption,char *initialText,int maxLen);

};
//------------------------------------------------------------------------------
/*static*/ bool CInputBox::Valid;
//------------------------------------------------------------------------------
CInputBox::CInputBox(const char *caption,char *text,int maxLen)
 : CWindow(ID_WINDOW,5,5,LCD_QueryWidth()-10,LCD_QueryHeight()-10,caption,TRUE,FALSE,FALSE),
   m_text(text),
   m_maxLen(maxLen)
{
  // create one line edit box & place buttons just below
  RECT rect;
  GetUsableRect(&rect); // get interior dimensions to calculate spacing
  int gap=10; // for nice visual spacing around objects
  m_textEdit=new CTextEdit(ID_TEXTEDIT,rect.width-2*gap,0,TEXTOPTION_ONELINE);
  m_textEdit->SetText(m_text);
  m_textEdit->SetFont(GetFont());
  m_textEdit->SetHeight(GUI_FontHeight(GetFont()));
  AddChild(m_textEdit,gap,gap);
  // now place buttons below - add custom keyboard icon for first button
  int y=m_textEdit->GetHeight()+2*gap;
  int buttonWidth=45, buttonHeight=17;
  AddChild(new CButton(ID_BUTTON_KEYBOARD,buttonWidth,buttonHeight,0,
           PKG_Image(GUI_pkg,GUI_KEYBOARD_OUT)),gap,y);
  AddChild(new CButton(ID_BUTTON_OK,      buttonWidth,buttonHeight,"OK"),
           rect.width-2*(buttonWidth+gap),y);
  AddChild(new CButton(ID_BUTTON_CANCEL,  buttonWidth,buttonHeight,"Cancel"),
           rect.width-buttonWidth-gap,y);
  // now resize and reposition dialog in display
  int frameVerticalSize=GetHeight()-rect.height;
  int newUsableHeight=y+buttonHeight+gap; // button position plus height and gap
  int newFullHeight=newUsableHeight+frameVerticalSize;
  SetHeight(newFullHeight);
  // now we center vertically
  SetY((LCD_QueryHeight()-newFullHeight)/2);
  // set focus to textbox (so all text goes there) and we're ready
  SetFocus(m_textEdit);
}
//------------------------------------------------------------------------------
S32 CInputBox::MsgHandler( MSG_TYPE type, CViewable *from, S32 data )
{
  switch (type)
  {
    case MSG_BUTTON_SELECT:
    {
      switch (from->GetID())
      {
        case ID_BUTTON_CANCEL: // no change to text - simply return
          Valid=false;
          Close();
          return 1;
        case ID_BUTTON_OK:
        {
          // ok pressed - get text into string buffer
          char *ptr=m_textEdit->GetText(); // create mem buff with text in it
          strncpy(m_text,ptr,m_maxLen);
          m_text[m_maxLen-1]='\0'; // copy/null temrinate
          free(ptr); // clean up
          Valid=true;
          Close();
        }
          return 1;
        case ID_BUTTON_KEYBOARD: // display keyboard dialog
          SetFocus(m_textEdit);  // make sure text goes to this control
          GUI_EventLoop(new CLatin1Keyboard(99));
          return 1;
      }
        break;
      default:
        break;
    }
  }
  return CWindow::MsgHandler(type, from, data);
}
//------------------------------------------------------------------------------
void CInputBox::Draw(void)
{
	CWindow::Draw();
	DrawChildren();
  // to make edit box more visible, we draw a black border aorund box
  RECT rect;
  m_textEdit->GetRect(&rect);
	DrawRect(rect.x-3,rect.y-3,rect.width+6,rect.height+6,COLOR_BLACK);
	DrawRect(rect.x-4,rect.y-4,rect.width+8,rect.height+8,COLOR_BLACK);
}
/*==============================================================================

       wrapper function for text entry dialog object

==============================================================================*/
bool InputBox(const char *caption,char *text,int maxLen)
{
  // puts up edit box fo rtext editing - text is source buffer, with maxLen the
  // total char avail (includig null position), and caption the box caption
  GUI_EventLoop(new CInputBox(caption,text,maxLen));
  return CInputBox::Valid;
}
//------------------------------------------------------------------------------

